# -*- coding: utf-8 -*-
"""Host Category Controller"""

import transaction

from tg import expose, flash, require, url, request, redirect, validate
from tg.decorators import override_template, tmpl_context
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from tgext.admin.tgadminconfig import TGAdminConfig
from tgext.admin.controller import AdminController
from repoze.what import predicates
from sqlalchemy import and_

from mirrormanager.lib.base import BaseController
from mirrormanager.model import DBSession, metadata
from mirrormanager.controllers.error import ErrorController
from mirrormanager import model
from mirrormanager.controllers.secure import SecureController

from mirrormanager import model
from mirrormanager.model import *

from mirrormanager.controllers.forms import *
from mirrormanager.controllers.utils import *


host_category_form_new = HostCategoryFormNew(submit_text="Save Host Category")

host_category_form_read = HostCategoryFormRead(submit_text="Save Host Category")


class HostCategoryController(BaseController):
    allow_only = predicates.not_anonymous()

    def disabled_fields(self, host=None):
        disabled_fields = []
        if not predicates.in_group(admin_group):
            disabled_fields.append('admin_active')
            disabled_fields.append('always_up2date')
        return disabled_fields

    def default(self, id):
        hc = model.DBSession.query(HostCategory).filter_by(id = id).one()
        downstream_siteadmin_check(hc.my_site(), identity)
        submit_action = url("/host_category/%s/update" % hostcategory.id)
        disabled_fields=self.disabled_fields()
        return dict(form=host_category_form_read, values=hc,
            action=submit_action, disabled_fields=self.disabled_fields(),
            host=hc.host)

    @expose(template="mirrormanager.templates.hostcategory")
    def new(self, **kwargs):
        try:
            hostid=kwargs['hostid']
            host = model.DBSession.query(Host).filter_by(id = hostid).one()
        except Exception, er:
            print "Error:", er
            raise redirect(url("/"))
        site = model.DBSession.query(Site).filter_by(id = host.site).one()
        siteadmin_check(site, request.identity['repoze.who.userid'])
        submit_action = url("/host_category/create?hostid=%s" % hostid)
        host.siteobj = site
        return dict(form=host_category_form_new, values=None,
                action=submit_action, disabled_fields=self.disabled_fields(),
                host=host)

    @expose(template="mirrormanager.templates.hostcategory")
    @validate(form=host_category_form_new)
    def create(self, **kwargs):

        if tmpl_context.form_errors:
            flash("Invalid input")
            return self.new(**kwargs)

        if not kwargs.has_key('hostid'):
            flash("Error: form did not provide hostid")
            raise redirect(url("/"))
        hostid = kwargs['hostid']
        del kwargs['hostid']

        try:
            host = model.DBSession.query(Host).filter_by(id = hostid).one()
        except Exception, er:
            print "Error:", er
            flash("Error: invalid hostid - foul play?")
            raise redirect(url("/"))
            
        try:
            category = Category.get(kwargs['category'])
        except Exception, er:
            print "Error:", er
            flash("Error: invalid category - foul play?")
            raise redirect(url("/host_category/new?hostid=%s" % hostid))
            
        del kwargs['category']

        try:
            hostcategory = HostCategory(host=host, category=category, **kwargs)
        except:
            flash("Error: Host already has category %s.  Try again." % category.name)
            raise redirect(url("/host_category/new?hostid=%s" % hostid))
        flash("HostCategory created.")
        raise redirect(url("/host_category/%s" % hostcategory.id))


    @expose(template="mirrormanager.templates.hostcategory")
    @validate(form=host_category_form_read)
    def update(self, hostcategory, tg_errors=None, **kwargs):

        if tmpl_context.form_errors:
            flash("Invalid input")
            return self.new(**kwargs)

        siteadmin_check(hostcategory.my_site(), identity)
        del kwargs['category']

        if tg_errors is not None:
            flash("Error updating HostCategory: %s" % createErrorString(tg_errors))
            submit_action = url("/host_category/%s/update" % hostcategory.id)
            return dict(form=host_category_form_read, values=hostcategory,
                action=submit_action, disabled_fields=self.disabled_fields(),
                host=hostcategory.host)
        
        
        hostcategory.set(**kwargs)
        hostcategory.sync()
        turbogears.flash("HostCategory Updated")
        raise turbogears.redirect(url("/"))

    @expose(template="mirrormanager.templates.hostcategory")
    def delete(self, hostcategory, **kwargs):
        siteadmin_check(hostcategory.my_site(), identity)
        hostid = hostcategory.host.id
        hostcategory.destroySelf()
        raise turbogears.redirect(url("/host/%s" % hostid))
