# -*- coding: utf-8 -*-
"""Host Category URL Controller"""

import transaction

from tg import expose, flash, require, url, request, redirect, validate
from tg.decorators import override_template, tmpl_context
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from tgext.admin.tgadminconfig import TGAdminConfig
from tgext.admin.controller import AdminController
from repoze.what import predicates
from sqlalchemy import and_

from mirrormanager.lib.base import BaseController
from mirrormanager.model import DBSession, metadata
from mirrormanager.controllers.error import ErrorController
from mirrormanager import model
from mirrormanager.controllers.secure import SecureController

from mirrormanager import model
from mirrormanager.model import *

from mirrormanager.controllers.forms import *
from mirrormanager.controllers.utils import *


host_category_url_form = HostCategoryUrlForm(submit_text="Create URL")

class HostCategoryUrlController(BaseController):
    allow_only = predicates.not_anonymous()
    page_title = "Host Category URL"
    form = host_category_url_form

    @expose(template="mirrormanager.templates.hostcategoryurl")
    def default(self, id):
        v = model.DBSession.query(HostCategoryUrl).filter_by(id = id).one()
        site = model.DBSession.query(Site).filter_by(id = v.site).one()
        downstream_siteadmin_check(site, identity)
        submit_action = url("/host_category_url/update/%s/" % hcurl.id)
        return dict(form=self.form, values=v, action=submit_action,
            page_title=self.page_title, host_category=v.host_category)
    
    @expose(template="mirrormanager.templates.hostcategoryurl")
    def new(self, **kwargs):
        try:
            hcid=kwargs['hostid']
            host_category = model.DBSession.query(HostCategory).filter_by(id = hcid).one()
        except Exception, er:
            print "Error:", er
            raise redirect(url("/"))

        host = host_category.host
        site = model.DBSession.query(Site).filter_by(id = host.site).one()
        siteadmin_check(site, request.identity['repoze.who.userid'])
            
        submit_action = url("/host_category_url/0/create?hcid=%s" % hcid)
        return dict(form=self.form, values=None, action=submit_action,
            page_title=self.page_title, host_category=host_category)

    @expose(template="mirrormanager.templates.hostcategoryurl")
    @validate(form=form)
    def create(self, **kwargs):
        if tmpl_context.form_errors:
            flash("Invalid input")
            return self.new(**kwargs)

        if not kwargs.has_key('hcid'):
            flash("Error: form didn't provide hcid")
            raise redirect("/")
        hcid = kwargs['hcid']

        try:
            hc = model.DBSession.query(HostCategory).filter_by(id = hcid).one()
        except Exception, er:
            print "Error:", er
            flash("Error: HostCategory %s does not exist" % hcid)
            raise redirect("/")

        site = model.DBSession.query(Site).filter_by(id = hc.site).one()
        siteadmin_check(site, request.identity['repoze.who.userid'])
        #siteadmin_check(hc.my_site(), identity)

        kwargs['url'] = strip(kwargs['url'])
        while kwargs['url'].endswith('/'):
            kwargs['url'] = strip(kwargs['url'][:-1])


        # This is ugly.  We've got this fake site 'Fedora Mirror'
        # which has all the hosts that didn't register themselves.
        # if someone now tries to register one, they get to the point
        # of making the hcurl, and it fails.  In this case, if the duplicate
        # belongs to a 'Fedora Mirror' site, great!  Nuke the host from
        # the Fedora Mirror site, and continue on.
        try:
            existing_hcurl = HostCategoryUrl.byUrl(kwargs['url'])
        except SQLObjectNotFound:
            pass
        else:
            otherhost = existing_hcurl.host_category.host
            othersite = otherhost.site
            if othersite.name == u'Fedora Mirror':
                otherhost.destroySelf()

        try:
            del kwargs['hcid']
            HostCategoryUrl(host_category=hc, **kwargs)
        except: # probably sqlite IntegrityError but we can't catch that for some reason... 
            flash("Error: entity already exists")
        flash("Success: HostCategoryURL created.")
        raise redirect(url("/host_category/%s" % hcid))

    @expose(template="mirrormanager.templates.hostcategoryurl")
    def update(self, hcurl, **kwargs):
        siteadmin_check(hcurl.my_site(), identity)
        if kwargs['url'].endswith('/'):
            kwargs['url'] = kwargs['url'][:-1]
        hcurl.set(**kwargs)
        hcurl.sync()
        submit_action = url("/host_category_url/%s/update" % hcurl.id)
        return dict(form=self.form, values=hcurl, action=submit_action,
            page_title=self.page_title, host_category=hcurl.host_category)


    @expose(template="mirrormanager.templates.hostcategoryurl")
    def delete(self, hcurl, **kwargs):
        hc = hcurl.host_category
        siteadmin_check(hcurl.my_site(), identity)
        hcurl.destroySelf()
        raise redirect(url("/host_category/%s" % hc.id))
