# -*- coding: utf-8 -*-
"""Main Controller"""

from tg import expose, flash, require, url, request, redirect, validate
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from tgext.admin.tgadminconfig import TGAdminConfig
from tgext.admin.controller import AdminController
from repoze.what import predicates

from mirrormanager.lib.base import BaseController
from mirrormanager.model import DBSession, metadata
from mirrormanager.controllers.error import ErrorController
from mirrormanager import model
from mirrormanager.controllers.secure import SecureController

from fedora.client import AuthError, AppError
from fedora.client.fas2 import AccountSystem

from mirrormanager import model
from mirrormanager.model import *

from mirrormanager.controllers.arch import *
from mirrormanager.controllers.countrycontinentredirect import *
from mirrormanager.controllers.embargoedcountry import *
from mirrormanager.controllers.host import *
from mirrormanager.controllers.hostaclip import *
from mirrormanager.controllers.hostcategory import *
from mirrormanager.controllers.hostcategoryurl import *
from mirrormanager.controllers.hostcountryallowed import *
from mirrormanager.controllers.hostnetblock import *
from mirrormanager.controllers.forms import *
from mirrormanager.controllers.product import *
from mirrormanager.controllers.repository import *
from mirrormanager.controllers.repositoryredirect import *
from mirrormanager.controllers.site import *
from mirrormanager.controllers.site2site import *
from mirrormanager.controllers.siteadmin import *
from mirrormanager.controllers.version import *

from mirrormanager.controllers.utils import *


__all__ = ['RootController', 'SiteController', 'SiteToSiteController',
            'HostController', 'SiteAdminController', 'HostCategoryUrlController',
            'HostCategoryController', 'ArchController'
        ]


class RootController(BaseController):
    """
    The root controller for the mirrormanager application.

    All the other controllers and WSGI applications should be mounted on this
    controller. For example::

        panel = ControlPanelController()
        another_app = AnotherWSGIApplication()

    Keep in mind that WSGI applications shouldn't be mounted directly: They
    must be wrapped around with :class:`tg.controllers.WSGIAppController`.

    """
    secc = SecureController()

    admin = AdminController(model, DBSession, config_type=TGAdminConfig)

    error = ErrorController()
    
    appTitle = 'Mirror manager'

    baseURL = "https://admin.fedoraproject.org/accounts"
    username = "elections"
    password = "elections2"
    fas = AccountSystem(baseURL, username=username, password=password)

    request.identity = request.environ.get('repoze.who.identity')
    
    site = SiteController()
    siteadmin = SiteAdminController()
    host = HostController()
    host_country_allowed = HostCountryAllowedController()
    host_acl_ip = HostAclIPController()
    host_netblock = HostNetblockController()
    host_category = HostCategoryController()
    host_category_url = HostCategoryUrlController()
    site2site = SiteToSiteController()
    product = ProductController()
    version = VersionController()
    arch = ArchController()
    embargoed_country = EmbargoedCountryController()
    repository = RepositoryController()
    repository_redirect = RepositoryRedirectController()
    country_continent_redirect = CountryContinentRedirectController()
    #from mirrormanager.xmlrpc import XmlrpcController
    #xmlrpc = XmlrpcController()

    @expose('mirrormanager.templates.welcome')
    @require(predicates.not_anonymous("Please log-in"))
    def index(self):
        """Handle the front-page."""
        if predicates.in_group(admin_group):
            sites = model.DBSession.query(Site).order_by('name').all()
            for siteobj in sites:
                siteobj.hosts = model.DBSession.query(Host).filter_by(site = siteobj.id).order_by('name').all()
        else:
            sites = []
            #sites = user_sites(userid)
        return dict(page='index', sites=sites)

    @expose('mirrormanager.templates.help')
    def help(self):
        """Handle the 'help' page."""
        return dict(page='help')

    @expose('mirrormanager.templates.environ')
    def environ(self):
        """This method showcases TG's access to the wsgi environment."""
        return dict(environment=request.environ)

    @expose('mirrormanager.templates.data')
    @expose('json')
    def data(self, **kw):
        """This method showcases how you can use the same controller for a data page and a display page"""
        return dict(params=kw)

    @expose('mirrormanager.templates.authentication')
    def auth(self):
        """Display some information about auth* on this application."""
        return dict(page='auth')

    @expose('mirrormanager.templates.index')
    @require(predicates.has_permission('manage', msg=l_('Only for managers')))
    def manage_permission_only(self, **kw):
        """Illustrate how a page for managers only works."""
        return dict(page='managers stuff')

    @expose('mirrormanager.templates.index')
    @require(predicates.is_user('editor', msg=l_('Only for the editor')))
    def editor_user_only(self, **kw):
        """Illustrate how a page exclusive for the editor works."""
        return dict(page='editor stuff')

    @expose('mirrormanager.templates.login')
    def login(self, came_from=url('/'), csrf_login = None):
        """Start the user login."""
        # code from elections v1
        #login_dict = fc_login(forward_url, args, kwargs)
        login_dict = {}
        login_dict['appTitle'] = '%s -- Fedora Account System Login' % \
                self.appTitle
        login_dict['message'] = _("Please log-in")
        login_dict['previous_url'] = came_from
        login_dict['forward_url'] = "/post_login"
        login_dict['original_parameters'] = request.params

        login_counter = request.environ['repoze.who.logins']
        if login_counter > 0:
            flash(_('Wrong credentials'), 'warning')
        if login_counter > 5:
            flash(_('Too many wront attempt -- Blocked user'), 'warning')
        login_dict['login_counter'] = str(login_counter)
        login_dict['page'] = 'login'
        login_dict['came_from'] = came_from

        return login_dict

    @expose()
    def post_login(self, came_from='/', *args, **kwargs):
        """
        Redirect the user to the initially requested page on successful
        authentication or redirect her back to the login page if login failed.
        """
        ## default code from TG2:
        if not request.identity:
            login_counter = request.environ['repoze.who.logins'] + 1
            redirect('/login', came_from=came_from, __logins=login_counter)
        userid = request.identity['repoze.who.userid']
        flash(_('Welcome back, %s!') % userid)
        redirect(came_from)

    @expose()
    def post_logout(self, came_from=url('/')):
        """
        Redirect the user to the initially requested page on logout and say
        goodbye as well.

        """
        flash(_('We hope to see you soon!'))
        redirect(came_from)
    
    ## End Of TG2's functions
    @expose(template="mirrormanager.templates.adminview")
    #@require(predicates.has_permission('manage', msg=l_('Only for managers')))
    def adminview(self):
        versions = model.DBSession.query(Version).order_by('name').all()
        for version in versions:
            version.productobj = model.DBSession.query(Product).filter_by(id = version.product).one()
        netblocks = model.DBSession.query(HostNetblock).order_by('host_id').all()
        for netblock in netblocks:
            netblock.hostobj = model.DBSession.query(Host).filter_by(id = netblock.host).one()
        return {"sites": model.DBSession.query(Site).order_by('name').all(),
                "arches": model.DBSession.query(Arch).order_by('name').all(),
                "products": model.DBSession.query(Product).order_by('name').all(),
                "versions": versions,
                "directories": model.DBSession.query(Directory).order_by('name').all(),
                "categories": model.DBSession.query(Category).order_by('name').all(),
                "repositories": model.DBSession.query(Repository).order_by('name').all(),
                "embargoed_countries": model.DBSession.query(EmbargoedCountry).all(),
                "netblocks": netblocks,
                "repository_redirects": model.DBSession.query(RepositoryRedirect).order_by('fromRepo').all(),
                "country_continent_redirects": model.DBSession.query(CountryContinentRedirect).order_by('country').all(),
                }
    
    @expose(template="mirrormanager.templates.rsync_acl",
            content_type="text/plain")
    def rsync_acl(self, **kwargs):
        internet2_only=False
        public_only=False
        if 'internet2_only' in kwargs:
            internet2_only = True
        if 'public_only' in kwargs:
            public_only = True

        result = rsync_acl_list(internet2_only=internet2_only, public_only=public_only)
        return dict(values=result)

    @expose(template="mirrormanager.templates.rsync_acl",
        content_type="text/plain")#format="plain", allow_json=True)
    def mirroradmins(self, **kwargs):
        if 'host' not in kwargs:
            return dict(values=[])
        host = kwargs['host']
        return dict(values=host_siteadmins(host))

    @expose(template="mirrormanager.templates.rsyncFilter",
        content_type="text/plain") # format="plain",
    def rsyncFilter(self, **kwargs):

        def parents(dir):
            p = []
            splitpath = dir.split('/')
            for i in xrange(1, len(splitpath)):
                p.append('/'.join(splitpath[:i]))
            return p

        def strip_prefix(to_strip, dir):
            splitdir = dir.split('/')
            splitdir = splitdir[to_strip:]
            dir = '/'.join(splitdir)
            return dir

        def num_prefix_components(prefix):
            splitprefix = prefix.split('/')
            return len(splitprefix)

        # by setting excludes here, we cause the filter rules to
        # not transfer anything if there is no new content or if
        # a mistake was made in the URL.
        message=None
        excludes=[u'*']
        try:
            c = kwargs['categories']
            since = kwargs['since']
            stripprefix = kwargs['stripprefix']
            num_prefix = num_prefix_components(stripprefix)
        except KeyError:
            message=u'Missing categories, since, or stripprefix arguments'
            return dict(includes=[], excludes=excludes, message=message)
        
        try:
            since = int(since)
        except:
            message=u'value of argument since is not an integer'
            return dict(includes=[], excludes=excludes, message=message)

        includes = set()
        categories_requested = c.split(',')
        newer_dirs = mirrormanager.model.rsyncFilter(categories_requested, since)
        for i in xrange(len(newer_dirs)):
            newer_dirs[i] = strip_prefix(num_prefix, newer_dirs[i])

        includes.update(newer_dirs)
        for n in newer_dirs:
            includes.update(parents(n))
        includes = list(includes)
        includes.sort()
        # add trailing slash as rsync wants it
        for i in xrange(len(includes)):
            includes[i] += u'/'
        return dict(includes=includes, excludes=excludes, message=message)
