# -*- coding: utf-8 -*-
"""Site Controller"""

import transaction

from tg import expose, flash, require, url, request, redirect, validate
from tg.decorators import override_template, tmpl_context
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from tgext.admin.tgadminconfig import TGAdminConfig
from tgext.admin.controller import AdminController
from repoze.what import predicates
from sqlalchemy import and_

from mirrormanager.lib.base import BaseController
from mirrormanager.model import DBSession, metadata
from mirrormanager.controllers.error import ErrorController
from mirrormanager.controllers.secure import SecureController

from mirrormanager import model
from mirrormanager.model import *

from mirrormanager.controllers.forms import *
from mirrormanager.controllers.utils import *

site_form = SiteForm("create_site_form",  action='create')

class SiteEntryController(object):
    allow_only = predicates.not_anonymous()

    def get(self, id, tg_errors=None, tg_source=None, **kwargs):
        site = model.DBSession.query(Site).filter_by(id = id).all()
        return dict(values=site, disabled_fields=self.disabled_fields(site=site))


class SiteController(BaseController):
    allow_only = predicates.not_anonymous()
    #site_entry_controller = SiteEntryController()

    @expose(template="mirrormanager.templates.site")
    def default(self, site):
        site = model.DBSession.query(Site).filter_by(id = site).one()
        site.admins = model.DBSession.query(SiteAdmin
                    ).filter_by(site = site.id).all()
        site.hosts = model.DBSession.query(Host
                    ).filter_by(site = site.id).all()
        ## FIXME: please rewrite this using subquery or so
        site.downstream_sites = []
        for s in model.DBSession.query(SiteToSite).filter_by(upstream_site = site.id).all():
            ds = model.DBSession.query(Site).filter_by(id = s.downstream_site).one()
            site.downstream_sites.append(ds)
        site.upstream_sites = []
        for s in model.DBSession.query(SiteToSite).filter_by(
                downstream_site = site.id).all():
            ups = model.DBSession.query(Site).filter_by(id = s.upstream_site).one() 
            site.upstream_sites.append(ups)
        downstream_siteadmin_check(site, request.identity['repoze.who.userid'])
        submit_action = url("/site/update/%s/" % site.id)
        return dict(form=site_form, values=site,
            action=submit_action,
            disabled_fields=disabled_fields(site=site))

    @expose(template="mirrormanager.templates.site")
    def new(self, **kwargs):
        submit_action = url("/site/create/")
        return dict(form=site_form, values=None,
            action=submit_action,
            disabled_fields=disabled_fields())

    @expose(template="mirrormanager.templates.site")
    @validate(form=site_form)
    def create(self, tg_errors=None, name=None, password=None,
                orgUrl=None, private=None, admin_active=None,
                user_active=None, allSitesCanPullFromMe=None,
                downstreamComments=None):
        if tmpl_context.form_errors:
            print tmpl_context.form_errors
            flash("Invalid input")
            #return self.new(**kwargs)

        if not predicates.in_group(admin_group) \
            and bool(admin_active):
            admin_active = None

        if tg_errors is not None:
            print tg_errors
            flash("Error creating Site: %s" % createErrorString(tg_errors))
            raise redirect(url("/site/0/create"))

        try:
            site = Site()
            site.name = name
            site.password = password
            site.orgUrl = orgUrl
            site.private = bool(private)
            site.admin_active = bool(admin_active)
            site.user_active = bool(user_active)
            site.createdBy = request.identity['repoze.who.userid']
            site.allSitesCanPullFromMe = bool(allSitesCanPullFromMe)
            site.downstreamComments = downstreamComments
            model.DBSession.add(site)
            transaction.commit()
            site = model.DBSession.query(Site).filter_by(name = name).one()
            sa = SiteAdmin(site=site.id, username=request.identity['repoze.who.userid'])
            model.DBSession.add(sa)
            transaction.commit()
            site = model.DBSession.query(Site).filter_by(name = name).one()
        except Exception, er: # probably sqlite IntegrityError but we can't catch that for some reason... 
            print "Error:", er
            flash("Error: Site %s already exists" % name)
            raise redirect(url("/site/new/"))
        else:

            flash("Site created.")
            raise redirect(url("/site/%s" % site.id))

    @expose(template="mirrormanager.templates.site")
    @validate(form=site_form)
    def update(self, site, tg_errors=None, name=None, password=None,
                orgUrl=None, private=None, admin_active=None,
                user_active=None, allSitesCanPullFromMe=None,
                downstreamComments=None):
        if tmpl_context.form_errors:
            print tmpl_context.form_errors
            flash("Invalid input")
            #return self.new(**kwargs)

        site = model.DBSession.query(Site).filter_by(id = site).one()
        siteadmin_check(site, request.identity['repoze.who.userid'])

        if tg_errors is not None:
            submit_action = url("/site/update/%s" % site.id)
            turbogears.flash("Error updating Site: %s" % createErrorString(tg_errors))
            return dict(form=site_form, values=site, action=submit_action,
                        disabled_fields=disabled_fields())

        if not predicates.in_group(admin_group) and bool(admin_active):
            admin_active = None
        
        ## Update the ORM
        info = {'name':name, 'password':password, 'orgUrl':orgUrl,
                'private':private, 'admin_active':bool(admin_active),
                'user_active':bool(user_active),
                'allSitesCanPullFromMe':bool(allSitesCanPullFromMe),
                'downstreamComments':downstreamComments}
        model.DBSession.query(Site).update(info) # sort out checkbox value
        flash("Site Updated")
        raise redirect(url("/site/%s" % site.id))

    @expose(template="mirrormanager.templates.site")
    def delete(self, site, **kwargs):
        site = model.DBSession.query(Site).filter_by(id = site).one()
        siteadmin_check(site, request.identity['repoze.who.userid'])
        model.DBSession.delete(site)
        sites = model.DBSession.query(SiteToSite).filter_by(upstream_site=site.id).all()
        for site in sites:
            model.DBSession.delete(site)
        sites = model.DBSession.query(SiteToSite).filter_by(downstream_site=site.id).all()
        for site in sites:
            model.DBSession.delete(site)
        raise redirect(url("/"))

    @expose(template="mirrormanager.templates.site")
    def s2s_delete(self, site, **kwargs):
        site = model.DBSession.query(Site).filter_by(id = site).one()
        siteadmin_check(site, request.identity['repoze.who.userid'])
        dsite = kwargs['dsite']
        del_downstream_site(site, dsite)
        raise redirect(url("/site/%s" % site.id))
