# -*- coding: utf-8 -*-
"""The application's model objects"""

from zope.sqlalchemy import ZopeTransactionExtension
from sqlalchemy.orm import scoped_session, sessionmaker
#from sqlalchemy import MetaData
from sqlalchemy.ext.declarative import declarative_base

# Global session manager: DBSession() returns the Thread-local
# session object appropriate for the current web request.
maker = sessionmaker(autoflush=True, autocommit=False,
                     extension=ZopeTransactionExtension())
DBSession = scoped_session(maker)

# Base class for all of our model classes: By default, the data model is
# defined with SQLAlchemy's declarative extension, but if you need more
# control, you can switch to the traditional method.
DeclarativeBase = declarative_base()

# There are two convenient ways for you to spare some typing.
# You can have a query property on all your model classes by doing this:
# DeclarativeBase.query = DBSession.query_property()
# Or you can use a session-aware mapper as it was used in TurboGears 1:
# DeclarativeBase = declarative_base(mapper=DBSession.mapper)

# Global metadata.
# The default metadata is the one from the declarative base.
metadata = DeclarativeBase.metadata

# If you have multiple databases with overlapping table names, you'll need a
# metadata for each database. Feel free to rename 'metadata2'.
#metadata2 = MetaData()

#####
# Generally you will not want to define your table's mappers, and data objects
# here in __init__ but will want to create modules them in the model directory
# and import them at the bottom of this file.
#
######

def init_model(engine):
    """Call me before using any of the tables or classes in the model."""

    DBSession.configure(bind=engine)
    # If you are using reflection to introspect your database and create
    # table objects for you, your tables must be defined and mapped inside
    # the init_model function, so that the engine is available if you
    # use the model outside tg2, you need to make sure this is called before
    # you use the model.

    #
    # See the following example:

    #global t_reflected

    #t_reflected = Table("Reflected", metadata,
    #    autoload=True, autoload_with=engine)

    #mapper(Reflected, t_reflected)

# Import your model modules here.
from mirrormanager.model.auth import User, Group, Permission
from mirrormanager.model.arch import Arch
from mirrormanager.model.category import Category
from mirrormanager.model.countrycontinentredirect import CountryContinentRedirect
from mirrormanager.model.directory import Directory
from mirrormanager.model.directoryexclusivehost import DirectoryExclusiveHost
from mirrormanager.model.embargoedcountry import EmbargoedCountry
from mirrormanager.model.filedetail import FileDetail
from mirrormanager.model.host import Host
from mirrormanager.model.hostaclip import HostAclIp
from mirrormanager.model.hostcategory import HostCategory
from mirrormanager.model.hostcategorydir import HostCategoryDir
from mirrormanager.model.hostcategoryurl import HostCategoryUrl
from mirrormanager.model.hostcountryallowed import HostCountryAllowed
from mirrormanager.model.hostnetblock import HostNetblock
from mirrormanager.model.hoststats import HostStats
from mirrormanager.model.product import Product
from mirrormanager.model.repository import Repository
from mirrormanager.model.repositoryredirect import RepositoryRedirect
from mirrormanager.model.site import Site
from mirrormanager.model.siteadmin import SiteAdmin
from mirrormanager.model.sitetosite import SiteToSite
from mirrormanager.model.version import Version
from mirrormanager.model.visit import Visit

def user_sites(identity):
    print Site.select(join=INNERJOINOn(
                            Site, SiteAdmin, 
                                AND(
                                    SiteAdmin.q.siteID == Site.q.id,
                                    SiteAdmin.q.username == identity
                                )
                            )
                        )
    return Site.select(join=INNERJOINOn(
                            Site, SiteAdmin, 
                                AND(
                                    SiteAdmin.q.siteID == Site.q.id,
                                    SiteAdmin.q.username == identity.current.user_name
                                )
                            )
                        )


def _rsync_acl_list(internet2_only, public_only):
    sql = "SELECT host_acl_ip.ip "
    sql += "FROM host, site, host_acl_ip "
    # join conditions
    sql += "WHERE "
    sql += "host.site_id = site.id AND "
    sql += "host_acl_ip.host_id = host.id "
    # select conditions
    # admin_active
    sql += 'AND host.admin_active AND site.admin_active '
    if internet2_only:
        sql += 'AND host.internet2 '
    if public_only:
        sql += 'AND NOT host.private '
        sql += 'AND NOT site.private '

    result = [t[0] for t in DBSession.execute(sql)]
    #result = Directory._connection.queryAll(sql)
    return result

def rsync_acl_list(internet2_only=False,public_only=False):
    result = _rsync_acl_list(internet2_only, public_only)
    return result
