# -*- coding: utf-8 -*-
"""Sample model module."""

from sqlalchemy import *
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import Integer, Unicode
#from sqlalchemy.orm import relation, backref
from datetime import datetime

from mirrormanager.model import DeclarativeBase, metadata, DBSession
from mirrormanager.model.siteadmin import SiteAdmin


class Site(DeclarativeBase):
    __tablename__ = 'site'

    id = Column(Integer, primary_key=True)
    name = Column(Text, unique=False, nullable=False)
    password = Column(Text, default=None)
    orgUrl = Column(Text, default=None)
    private = Column(Boolean, default=False)
    admin_active = Column(Boolean, default=True)
    user_active  = Column(Boolean, default=True)
    createdAt = Column(DateTime, default=datetime.utcnow())
    createdBy = Column(Text, default=None)
    # allow all sites to pull from me
    allSitesCanPullFromMe = Column(Boolean, default=False)
    downstreamComments = Column(Text, default=None)
    
    #admins = MultipleJoin('SiteAdmin')
    #hosts  = MultipleJoin('Host')

    def destroySelf(self):
        """Cascade the delete operation"""
        for h in self.hosts:
            h.destroySelf()
        for a in self.admins:
            a.destroySelf()
        for s in SiteToSite.select(OR(SiteToSite.q.upstream_siteID == self.id,
                                        SiteToSite.q.downstream_siteID == self.id)):
            s.destroySelf()
        SQLObject.destroySelf(self)

    def _get_downstream_sites(self):
        if self.allSitesCanPullFromMe:
            return [s for s in Site.select() if s != self]
        else:
            return [s2s.downstream_site for s2s in SiteToSite.selectBy(upstream_site=self)]

    def _get_upstream_sites(self):
        open_upstreams   = [s for s in Site.select() if s != self and s.allSitesCanPullFromMe]
        chosen_upstreams = [s2s.upstream_site for s2s in SiteToSite.selectBy(downstream_site=self)]
        result = uniqueify(open_upstreams + chosen_upstreams)
        return result

    def add_downstream_site(self, site):
        if site is not None:
            SiteToSite(upstream_site=self, downstream_site=site)

        
    def is_siteadmin_byname(self, name):
        for a in DBSession.query(SiteAdmin).all():
            if a.site == self.id and a.username == name:
                return True
        return False

    def is_siteadmin(self, identity):
        admin_group = []
        # FIXME: was: config.get('mirrormanager.admin_group', 'sysadmin')
        if identity in admin_group:
            return True
        return self.is_siteadmin_byname(identity)

    
    def is_downstream_siteadmin_byname(self, name):
        #for d in self.downstream_sites:
            #for a in d.admins:
                #if a.username == name:
                    #return True
        return False

    def is_downstream_siteadmin(self, identity):
        """If you are a sysadmin of one of my immediate downstream sites,
        you can see some of my site details, but you can't edit them.
        """
        return self.is_downstream_siteadmin_byname(identity)

