# -*- coding: utf-8 -*-
"""Sample model module."""

from sqlalchemy import *
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import Integer, Unicode
#from sqlalchemy.orm import relation, backref

from mirrormanager.model import DeclarativeBase, metadata, DBSession


class User(DeclarativeBase):
    """
    Reasonably basic User definition. 
    Probably would want additional attributes.
    """
    # names like "Group", "Order" and "User" are reserved words in SQL
    # so we set the name to something safe for SQL
    __tablename__ = 'tg_user'

    id = Column(Integer, primary_key=True)
    user_name = Column(Text, length=16)
    email_address = Column(Text, length=255)
    display_name = Column(Text, length=255)
    password = Column(Text, length=40)
    created = Column(DateTime, default=datetime.now)

    # groups this user belongs to
    #groups = RelatedJoin("Group", intermediateTable="user_group",
                         #joinColumn="user_id", otherColumn="group_id")

    def _get_permissions(self):
        perms = set()
        for g in self.groups:
            perms = perms | set(g.permissions)
        return perms

    def _set_password(self, cleartext_password):
        "Runs cleartext_password through the hash algorithm before saving."
        hash = identity.encrypt_password(cleartext_password)
        self._SO_set_password(hash)

    def set_password_raw(self, password):
        "Saves the password as-is to the database."
        self._SO_set_password(password)
